package cn.com.infosec.jce;

import cn.com.infosec.asn1.ASN1OctetString;
import cn.com.infosec.asn1.ASN1Sequence;
import cn.com.infosec.asn1.ASN1Set;
import cn.com.infosec.asn1.DERConstructedSequence;
import cn.com.infosec.asn1.DERConstructedSet;
import cn.com.infosec.asn1.DEREncodable;
import cn.com.infosec.asn1.DERInputStream;
import cn.com.infosec.asn1.DERInteger;
import cn.com.infosec.asn1.DERObject;
import cn.com.infosec.asn1.DERObjectIdentifier;
import cn.com.infosec.asn1.DEROctetString;
import cn.com.infosec.asn1.DEROutputStream;
import cn.com.infosec.asn1.DERTaggedObject;
import cn.com.infosec.asn1.pkcs.ContentInfo;
import cn.com.infosec.asn1.pkcs.IssuerAndSerialNumber;
import cn.com.infosec.asn1.pkcs.PKCSObjectIdentifiers;
import cn.com.infosec.asn1.pkcs.SignedData;
import cn.com.infosec.asn1.pkcs.SignerInfo;
import cn.com.infosec.asn1.x509.AlgorithmIdentifier;
import cn.com.infosec.asn1.x509.CertificateList;
import cn.com.infosec.asn1.x509.DigestInfo;
import cn.com.infosec.asn1.x509.X509CertificateStructure;
import cn.com.infosec.asn1.x509.X509Name;
import cn.com.infosec.asn1.x509.X509ObjectIdentifiers;
import cn.com.infosec.crypto.AsymmetricBlockCipher;
import cn.com.infosec.crypto.Digest;
import cn.com.infosec.crypto.digests.SHA1Digest;
import cn.com.infosec.crypto.encodings.PKCS1Encoding;
import cn.com.infosec.crypto.engines.RSAEngine;
import cn.com.infosec.crypto.params.RSAKeyParameters;
import cn.com.infosec.jce.provider.RSAUtil;
import cn.com.infosec.jce.provider.X509CRLObject;
import cn.com.infosec.jce.provider.X509CertificateObject;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.math.BigInteger;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.Signature;
import java.security.SignatureException;
import java.security.cert.CRL;
import java.security.cert.Certificate;
import java.security.cert.X509CRL;
import java.security.cert.X509Certificate;
import java.security.interfaces.RSAPrivateKey;
import java.security.interfaces.RSAPublicKey;
import java.security.spec.AlgorithmParameterSpec;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

public class PKCS7SignedDataICBC implements PKCSObjectIdentifiers {
   private int version;
   private int signerversion;
   private Set digestalgos;
   private Collection certs;
   private Collection crls;
   private X509Certificate signCert;
   private byte[] digest;
   private String digestAlgorithm;
   private String digestEncryptionAlgorithm;
   private SignatureICBC sig;
   private transient PrivateKey privKey;
   private byte[] contentData;
   private final String ID_PKCS7_DATA;
   private final String ID_PKCS7_SIGNED_DATA;
   private final String ID_MD5;
   private final String ID_MD2;
   private final String ID_SHA1;
   private final String ID_RSA;
   private final String ID_DSA;

   public PKCS7SignedDataICBC(byte[] in, byte[] hash) {
      this(in, hash, "INFOSEC");
   }

   public PKCS7SignedDataICBC(byte[] in, byte[] hash, String provider) {
      this.ID_PKCS7_DATA = "1.2.840.113549.1.7.1";
      this.ID_PKCS7_SIGNED_DATA = "1.2.840.113549.1.7.2";
      this.ID_MD5 = "1.2.840.113549.2.5";
      this.ID_MD2 = "1.2.840.113549.2.2";
      this.ID_SHA1 = "1.3.14.3.2.26";
      this.ID_RSA = "1.2.840.113549.1.1.1";
      this.ID_DSA = "1.2.840.10040.4.1";
      DERInputStream var10000 = new DERInputStream(new ByteArrayInputStream(in));

      DERObject var20;
      try {
         var20 = var10000.readObject();
      } catch (IOException var6) {
         throw new SecurityException("can't decode PKCS7SignedData object");
      }

      DERObject in1 = var20;
      if (var20 instanceof DERConstructedSequence) {
         ContentInfo in2;
         if (!(in2 = ContentInfo.getInstance(in1)).getContentType().equals(PKCSObjectIdentifiers.signedData)) {
            throw new SecurityException("Not a valid PKCS#7 signed-data object - wrong header " + in2.getContentType().getId());
         } else {
            SignedData hash1 = SignedData.getInstance(in2.getContent());
            this.certs = new ArrayList();
            Enumeration in3;
            if (hash1.getCertificates() != null) {
               in3 = ASN1Set.getInstance(hash1.getCertificates()).getObjects();

               while(in3.hasMoreElements()) {
                  this.certs.add(new X509CertificateObject(X509CertificateStructure.getInstance(in3.nextElement())));
               }
            }

            this.crls = new ArrayList();
            if (hash1.getCRLs() != null) {
               in3 = ASN1Set.getInstance(hash1.getCRLs()).getObjects();

               while(in3.hasMoreElements()) {
                  this.crls.add(new X509CRLObject(CertificateList.getInstance(in3.nextElement())));
               }
            }

            this.version = hash1.getVersion().getValue().intValue();
            DEREncodable in4;
            if ((in4 = hash1.getContentInfo().getContent()) != null) {
               this.contentData = ((ASN1OctetString)in4).getOctets();
            } else {
               this.contentData = null;
            }

            this.digestalgos = new HashSet();
            Enumeration provider1 = hash1.getDigestAlgorithms().getObjects();

            while(provider1.hasMoreElements()) {
               DERObjectIdentifier in5 = (DERObjectIdentifier)((DERConstructedSequence)provider1.nextElement()).getObjectAt(0);
               this.digestalgos.add(in5.getId());
            }

            DERConstructedSet in6;
            if ((in6 = (DERConstructedSet)hash1.getSignerInfos()).getSize() != 1) {
               throw new SecurityException("This PKCS#7 object has multiple SignerInfos - only one is supported at this time");
            } else {
               SignerInfo in7;
               SignerInfo var10001 = in7 = SignerInfo.getInstance(in6.getObjectAt(0));
               this.signerversion = in7.getVersion().getValue().intValue();
               IssuerAndSerialNumber hash2;
               BigInteger provider2 = (hash2 = var10001.getIssuerAndSerialNumber()).getCertificateSerialNumber().getValue();
               X509Principal var4;
               var4 = new X509Principal.<init>(hash2.getName());
               Iterator hash3 = this.certs.iterator();

               while(hash3.hasNext()) {
                  X509Certificate var5;
                  if (provider2.equals((var5 = (X509Certificate)hash3.next()).getSerialNumber()) && var4.equals(var5.getIssuerDN())) {
                     this.signCert = var5;
                     break;
                  }
               }

               if (this.signCert != null) {
                  this.digestAlgorithm = in7.getDigestAlgorithm().getObjectId().getId();
                  this.digest = in7.getEncryptedDigest().getOctets();
                  this.digestEncryptionAlgorithm = in7.getDigestEncryptionAlgorithm().getObjectId().getId();
                  SignatureICBC in8;
                  SignatureICBC var21 = in8 = new SignatureICBC;
                  in8.<init>();
                  this.sig = in8;
                  var21.engineInitVerify(this.signCert.getPublicKey());
               } else {
                  throw new SecurityException("Can't find signing certificate with serial " + provider2.toString(16));
               }
            }
         }
      } else {
         throw new SecurityException("Not a valid PKCS#7 object - not a sequence");
      }
   }

   public PKCS7SignedDataICBC(PrivateKey privKey, Certificate[] certChain, String hashAlgorithm) {
      this(privKey, certChain, hashAlgorithm, "INFOSEC");
   }

   public PKCS7SignedDataICBC(PrivateKey privKey, Certificate[] certChain, String hashAlgorithm, String provider) {
      this(privKey, certChain, (CRL[])null, hashAlgorithm, provider);
   }

   public PKCS7SignedDataICBC(PrivateKey privKey, Certificate[] certChain, CRL[] crlList, String hashAlgorithm, String provider) {
      this.ID_PKCS7_DATA = "1.2.840.113549.1.7.1";
      this.ID_PKCS7_SIGNED_DATA = "1.2.840.113549.1.7.2";
      this.ID_MD5 = "1.2.840.113549.2.5";
      this.ID_MD2 = "1.2.840.113549.2.2";
      this.ID_SHA1 = "1.3.14.3.2.26";
      this.ID_RSA = "1.2.840.113549.1.1.1";
      this.ID_DSA = "1.2.840.10040.4.1";
      this.privKey = privKey;
      if (hashAlgorithm.equals("MD5")) {
         this.digestAlgorithm = "1.2.840.113549.2.5";
      } else if (hashAlgorithm.equals("MD2")) {
         this.digestAlgorithm = "1.2.840.113549.2.2";
      } else if (hashAlgorithm.equals("SHA")) {
         this.digestAlgorithm = "1.3.14.3.2.26";
      } else {
         if (!hashAlgorithm.equals("SHA1")) {
            throw new NoSuchAlgorithmException("Unknown Hash Algorithm " + hashAlgorithm);
         }

         this.digestAlgorithm = "1.3.14.3.2.26";
      }

      this.signerversion = 1;
      this.version = 1;
      this.certs = new ArrayList();
      this.crls = new ArrayList();
      HashSet hashAlgorithm1;
      HashSet var10002 = hashAlgorithm1 = new HashSet;
      hashAlgorithm1.<init>();
      this.digestalgos = hashAlgorithm1;
      var10002.add(this.digestAlgorithm);
      this.signCert = (X509Certificate)certChain[0];

      for(int hashAlgorithm2 = 0; hashAlgorithm2 < certChain.length; ++hashAlgorithm2) {
         this.certs.add(certChain[hashAlgorithm2]);
      }

      if (crlList != null) {
         for(int certChain1 = 0; certChain1 < crlList.length; ++certChain1) {
            this.crls.add(crlList[certChain1]);
         }
      }

      if ((this.digestEncryptionAlgorithm = privKey.getAlgorithm()).equals("RSA")) {
         this.digestEncryptionAlgorithm = "1.2.840.113549.1.1.1";
      } else {
         if (!this.digestEncryptionAlgorithm.equals("DSA")) {
            throw new NoSuchAlgorithmException("Unknown Key Algorithm " + this.digestEncryptionAlgorithm);
         }

         this.digestEncryptionAlgorithm = "1.2.840.10040.4.1";
      }

      SignatureICBC certChain2;
      SignatureICBC var10000 = certChain2 = new SignatureICBC;
      certChain2.<init>();
      this.sig = certChain2;
      var10000.initSign(privKey);
   }

   private DERObject getIssuer(byte[] enc) {
      IOException var10000;
      label32: {
         boolean var10001;
         DERConstructedSequence var8;
         try {
            var8 = (DERConstructedSequence)(new DERInputStream(new ByteArrayInputStream(enc))).readObject();
         } catch (IOException var4) {
            var10000 = var4;
            var10001 = false;
            break label32;
         }

         DERConstructedSequence var5 = var8;

         boolean var9;
         try {
            var9 = var8.getObjectAt(0) instanceof DERTaggedObject;
         } catch (IOException var3) {
            var10000 = var3;
            var10001 = false;
            break label32;
         }

         byte enc1;
         if (var9) {
            enc1 = 3;
         } else {
            enc1 = 2;
         }

         try {
            return (DERObject)var5.getObjectAt(enc1);
         } catch (IOException var2) {
            var10000 = var2;
            var10001 = false;
         }
      }

      IOException var6 = var10000;
      throw new Error("IOException reading from ByteArray: " + var6);
   }

   public String getDigestAlgorithm() {
      String var1;
      String var10000 = var1 = this.digestAlgorithm;
      String var2 = this.digestEncryptionAlgorithm;
      if (var10000.equals("1.2.840.113549.2.5")) {
         var1 = "MD5";
      } else if (this.digestAlgorithm.equals("1.2.840.113549.2.2")) {
         var1 = "MD2";
      } else if (this.digestAlgorithm.equals("1.3.14.3.2.26")) {
         var1 = "SHA1";
      }

      if (this.digestEncryptionAlgorithm.equals("1.2.840.113549.1.1.1")) {
         var2 = "RSA";
      } else if (this.digestEncryptionAlgorithm.equals("1.2.840.10040.4.1")) {
         var2 = "DSA";
      }

      return var1 + "with" + var2;
   }

   public void reset() {
      Exception var10000;
      label28: {
         boolean var10001;
         PrivateKey var7;
         try {
            var7 = this.privKey;
         } catch (Exception var5) {
            var10000 = var5;
            var10001 = false;
            break label28;
         }

         PrivateKey var1 = var7;
         if (var7 == null) {
            try {
               this.sig.initVerify(this.signCert.getPublicKey());
            } catch (Exception var4) {
               var10000 = var4;
               var10001 = false;
               break label28;
            }
         } else {
            try {
               this.sig.initSign(var1);
            } catch (Exception var3) {
               var10000 = var3;
               var10001 = false;
               break label28;
            }
         }

         try {
            return;
         } catch (Exception var2) {
            var10000 = var2;
            var10001 = false;
         }
      }

      Exception var6 = var10000;
      throw new RuntimeException(var6.toString());
   }

   public Certificate[] getCertificates() {
      return (X509Certificate[])this.certs.toArray(new X509Certificate[0]);
   }

   public Collection getCRLs() {
      return this.crls;
   }

   public X509Certificate getSigningCertificate() {
      return this.signCert;
   }

   public int getVersion() {
      return this.version;
   }

   public int getSigningInfoVersion() {
      return this.signerversion;
   }

   public byte[] getContentData() {
      return this.contentData;
   }

   public void update(byte buf) {
      this.sig.update(buf);
   }

   public void update(byte[] buf, int off, int len) {
      this.sig.update(buf, off, len);
   }

   public boolean verify() {
      return this.sig.verify(this.digest);
   }

   public boolean verify(byte[] hash) {
      return this.sig.engineVerify(hash, this.digest);
   }

   public byte[] getEncoded() {
      return this.getEncoded((byte[])null);
   }

   public byte[] getEncoded(byte[] _contentData) {
      Exception var10000;
      label202: {
         boolean var10001;
         byte[] var42;
         try {
            var42 = this.digest;
         } catch (Exception var32) {
            var10000 = var32;
            var10001 = false;
            break label202;
         }

         if (var42 == null) {
            try {
               this.digest = this.sig.sign();
            } catch (Exception var31) {
               var10000 = var31;
               var10001 = false;
               break label202;
            }
         }

         PKCS7SignedDataICBC var43;
         DERConstructedSet var44;
         try {
            var43 = this;
            var44 = new DERConstructedSet;
         } catch (Exception var30) {
            var10000 = var30;
            var10001 = false;
            break label202;
         }

         DERConstructedSet var2 = var44;

         Iterator var45;
         try {
            var44.<init>();
            var45 = var43.digestalgos.iterator();
         } catch (Exception var29) {
            var10000 = var29;
            var10001 = false;
            break label202;
         }

         Iterator var3 = var45;

         label185:
         while(true) {
            boolean var46;
            try {
               var46 = var3.hasNext();
            } catch (Exception var27) {
               var10000 = var27;
               var10001 = false;
               break;
            }

            if (!var46) {
               DERConstructedSequence var49;
               try {
                  var42 = _contentData;
                  var49 = new DERConstructedSequence;
               } catch (Exception var26) {
                  var10000 = var26;
                  var10001 = false;
                  break;
               }

               DERConstructedSequence var39;
               DERConstructedSequence var10002 = var39 = var49;

               try {
                  var10002.<init>();
                  var49.addObject(new DERObjectIdentifier("1.2.840.113549.1.7.1"));
               } catch (Exception var25) {
                  var10000 = var25;
                  var10001 = false;
                  break;
               }

               DERConstructedSequence var52;
               if (var42 != null) {
                  DEROctetString var53;
                  try {
                     var52 = var39;
                     var53 = new DEROctetString;
                  } catch (Exception var24) {
                     var10000 = var24;
                     var10001 = false;
                     break;
                  }

                  DEROctetString var4 = var53;

                  try {
                     var53.<init>(_contentData);
                     var52.addObject(new DERTaggedObject(0, var4));
                  } catch (Exception var23) {
                     var10000 = var23;
                     var10001 = false;
                     break;
                  }
               }

               try {
                  var43 = this;
                  var44 = new DERConstructedSet;
               } catch (Exception var22) {
                  var10000 = var22;
                  var10001 = false;
                  break;
               }

               DERConstructedSet _contentData1 = var44;

               try {
                  var44.<init>();
                  var45 = var43.certs.iterator();
               } catch (Exception var21) {
                  var10000 = var21;
                  var10001 = false;
                  break;
               }

               Iterator var40 = var45;

               while(true) {
                  try {
                     var46 = var40.hasNext();
                  } catch (Exception var19) {
                     var10000 = var19;
                     var10001 = false;
                     break label185;
                  }

                  if (!var46) {
                     try {
                        var43 = this;
                        var49 = new DERConstructedSequence;
                     } catch (Exception var18) {
                        var10000 = var18;
                        var10001 = false;
                        break label185;
                     }

                     var10002 = var49;
                     DERConstructedSequence var10003 = var49;
                     DERConstructedSequence var10004 = var49;
                     DERConstructedSequence var10005 = var49;
                     DERConstructedSequence var41;
                     DERConstructedSequence var10006 = var41 = var49;

                     try {
                        var10006.<init>();
                        var10005.addObject(new DERInteger(this.signerversion));
                        var10004.addObject(new IssuerAndSerialNumber(new X509Name((DERConstructedSequence)this.getIssuer(this.signCert.getTBSCertificate())), new DERInteger(this.signCert.getSerialNumber())));
                        var10003.addObject(new AlgorithmIdentifier(new DERObjectIdentifier(this.digestAlgorithm), (DEREncodable)null));
                        var10002.addObject(new AlgorithmIdentifier(new DERObjectIdentifier(this.digestEncryptionAlgorithm), (DEREncodable)null));
                        var49.addObject(new DEROctetString(this.digest));
                        var49 = new DERConstructedSequence;
                     } catch (Exception var17) {
                        var10000 = var17;
                        var10001 = false;
                        break label185;
                     }

                     DERConstructedSequence var5;
                     var10002 = var5 = var49;

                     int var54;
                     try {
                        var5.<init>();
                        var5.addObject(new DERInteger(this.version));
                        var5.addObject(var2);
                        var10002.addObject(var39);
                        var49.addObject(new DERTaggedObject(false, 0, _contentData1));
                        var54 = var43.crls.size();
                     } catch (Exception var16) {
                        var10000 = var16;
                        var10001 = false;
                        break label185;
                     }

                     if (var54 > 0) {
                        try {
                           var43 = this;
                           var44 = new DERConstructedSet;
                        } catch (Exception var15) {
                           var10000 = var15;
                           var10001 = false;
                           break label185;
                        }

                        DERConstructedSet var33 = var44;

                        try {
                           var44.<init>();
                           var45 = var43.crls.iterator();
                        } catch (Exception var14) {
                           var10000 = var14;
                           var10001 = false;
                           break label185;
                        }

                        Iterator _contentData2 = var45;

                        while(true) {
                           try {
                              var46 = _contentData2.hasNext();
                           } catch (Exception var12) {
                              var10000 = var12;
                              var10001 = false;
                              break label185;
                           }

                           if (!var46) {
                              try {
                                 var5.addObject(new DERTaggedObject(false, 1, var33));
                                 break;
                              } catch (Exception var11) {
                                 var10000 = var11;
                                 var10001 = false;
                                 break label185;
                              }
                           }

                           try {
                              var33.addObject((new DERInputStream(new ByteArrayInputStream(((X509CRL)_contentData2.next()).getEncoded()))).readObject());
                           } catch (Exception var13) {
                              var10000 = var13;
                              var10001 = false;
                              break label185;
                           }
                        }
                     }

                     try {
                        var52 = var5;
                        var44 = new DERConstructedSet;
                     } catch (Exception var10) {
                        var10000 = var10;
                        var10001 = false;
                        break label185;
                     }

                     DERConstructedSet var47 = var44;
                     DERConstructedSet var48 = var44;

                     try {
                        var48.<init>();
                        var47.addObject(var41);
                        var52.addObject(var44);
                        var52 = new DERConstructedSequence;
                     } catch (Exception var9) {
                        var10000 = var9;
                        var10001 = false;
                        break label185;
                     }

                     var49 = var52;
                     DERConstructedSequence var34;
                     var10002 = var34 = var52;

                     ByteArrayOutputStream var57;
                     try {
                        var10002.<init>();
                        var49.addObject(new DERObjectIdentifier("1.2.840.113549.1.7.2"));
                        var52.addObject(new DERTaggedObject(0, var5));
                        var57 = new ByteArrayOutputStream;
                     } catch (Exception var8) {
                        var10000 = var8;
                        var10001 = false;
                        break label185;
                     }

                     ByteArrayOutputStream _contentData3;
                     ByteArrayOutputStream var55 = _contentData3 = var57;

                     DEROutputStream var56;
                     try {
                        var55.<init>();
                        var56 = new DEROutputStream;
                     } catch (Exception var7) {
                        var10000 = var7;
                        var10001 = false;
                        break label185;
                     }

                     DEROutputStream var50 = var56;
                     DEROutputStream var51 = var56;

                     try {
                        var51.<init>(_contentData3);
                        var50.writeObject(var34);
                        var56.close();
                        return var57.toByteArray();
                     } catch (Exception var6) {
                        var10000 = var6;
                        var10001 = false;
                        break label185;
                     }
                  }

                  try {
                     _contentData1.addObject((new DERInputStream(new ByteArrayInputStream(((X509Certificate)var40.next()).getEncoded()))).readObject());
                  } catch (Exception var20) {
                     var10000 = var20;
                     var10001 = false;
                     break label185;
                  }
               }
            }

            try {
               var2.addObject(new AlgorithmIdentifier(new DERObjectIdentifier((String)var3.next()), (DEREncodable)null));
            } catch (Exception var28) {
               var10000 = var28;
               var10001 = false;
               break;
            }
         }
      }

      Exception var35 = var10000;
      throw new RuntimeException(var35.toString());
   }

   class SignatureICBC extends Signature implements PKCSObjectIdentifiers, X509ObjectIdentifiers {
      private Digest digest = new SHA1Digest();
      private AsymmetricBlockCipher cipher = new PKCS1Encoding(new RSAEngine());
      private AlgorithmIdentifier algId;
      private byte[] hash;

      public SignatureICBC() {
         super("SHA1withRSA");
         this.algId = new AlgorithmIdentifier(X509ObjectIdentifiers.id_SHA1, (DEREncodable)null);
      }

      private byte[] derEncode(byte[] hash) {
         ByteArrayOutputStream var2;
         ByteArrayOutputStream var10000 = var2 = new ByteArrayOutputStream;
         var10000.<init>();
         (new DEROutputStream(var2)).writeObject(new DigestInfo(this.algId, hash));
         return var10000.toByteArray();
      }

      private DigestInfo derDecode(byte[] encoding) {
         ByteArrayInputStream var2;
         var2 = new ByteArrayInputStream.<init>(encoding);
         DERInputStream encoding1;
         encoding1 = new DERInputStream.<init>(var2);
         return new DigestInfo((ASN1Sequence)encoding1.readObject());
      }

      protected void engineInitVerify(PublicKey publicKey) {
         if (publicKey instanceof RSAPublicKey) {
            SignatureICBC var10000 = this;
            SignatureICBC var10001 = this;
            RSAKeyParameters var2 = RSAUtil.generatePublicKeyParameter((RSAPublicKey)publicKey);
            var10001.digest.reset();
            var10000.cipher.init(false, var2);
         } else {
            throw new InvalidKeyException("Supplied key is not a RSAPublicKey instance");
         }
      }

      protected void engineInitSign(PrivateKey privateKey) {
         if (privateKey instanceof RSAPrivateKey) {
            SignatureICBC var10000 = this;
            SignatureICBC var10001 = this;
            RSAKeyParameters var2 = RSAUtil.generatePrivateKeyParameter((RSAPrivateKey)privateKey);
            var10001.digest.reset();
            var10000.cipher.init(true, var2);
         } else {
            throw new InvalidKeyException("Supplied key is not a RSAPrivateKey instance");
         }
      }

      protected void engineUpdate(byte b) {
         this.digest.update(b);
      }

      protected void engineUpdate(byte[] b, int off, int len) {
         this.digest.update(b, off, len);
      }

      protected byte[] engineSign() {
         SignatureICBC var10000 = this;
         SignatureICBC var10001 = this;
         byte[] var10002 = new byte[this.digest.getDigestSize()];
         this.digest.doFinal(var10002, 0);

         Exception var10;
         label49: {
            boolean var12;
            byte[] var13;
            try {
               var13 = var10001.derEncode(var10002);
            } catch (ArrayIndexOutOfBoundsException var5) {
               var12 = false;
               throw new SignatureException("key too small for signature type");
            } catch (Exception var6) {
               var10 = var6;
               var12 = false;
               break label49;
            }

            byte[] var7 = var13;

            AsymmetricBlockCipher var11;
            try {
               var11 = var10000.cipher;
            } catch (ArrayIndexOutOfBoundsException var3) {
               var12 = false;
               throw new SignatureException("key too small for signature type");
            } catch (Exception var4) {
               var10 = var4;
               var12 = false;
               break label49;
            }

            var13 = var7;
            var10002 = var7;
            byte var8 = 0;

            try {
               return var11.processBlock(var13, var8, var10002.length);
            } catch (ArrayIndexOutOfBoundsException var1) {
               var12 = false;
               throw new SignatureException("key too small for signature type");
            } catch (Exception var2) {
               var10 = var2;
               var12 = false;
            }
         }

         Exception var9 = var10;
         throw new SignatureException(var9.toString());
      }

      protected boolean engineVerify(byte[] sigBytes) {
         SignatureICBC var10000 = this;
         SignatureICBC var10001 = this;
         int var2;
         byte[] var3 = new byte[var2 = this.digest.getDigestSize()];
         this.digest.doFinal(var3, 0);

         boolean var10;
         AsymmetricBlockCipher var11;
         try {
            var11 = var10001.cipher;
         } catch (Exception var5) {
            var10 = false;
            return false;
         }

         byte[] var10002 = sigBytes;
         byte[] var10003 = sigBytes;
         int sigBytes1 = 0;

         DigestInfo var9;
         try {
            var9 = var10000.derDecode(var11.processBlock(var10002, sigBytes1, var10003.length));
         } catch (Exception var4) {
            var10 = false;
            return false;
         }

         DigestInfo sigBytes2 = var9;
         if (!var9.getAlgorithmId().equals(this.algId)) {
            return false;
         } else {
            byte[] var6;
            if (var2 != (var6 = sigBytes2.getDigest()).length) {
               return false;
            } else {
               for(sigBytes1 = 0; sigBytes1 < var2; ++sigBytes1) {
                  if (var6[sigBytes1] != var3[sigBytes1]) {
                     return false;
                  }
               }

               return true;
            }
         }
      }

      protected boolean engineVerify(byte[] param1, byte[] param2) {
         // $FF: Couldn't be decompiled
      }

      protected void engineSetParameter(AlgorithmParameterSpec params) {
         throw new UnsupportedOperationException("engineSetParameter unsupported");
      }

      /** @deprecated */
      protected void engineSetParameter(String param, Object value) {
         throw new UnsupportedOperationException("engineSetParameter unsupported");
      }

      /** @deprecated */
      protected Object engineGetParameter(String param) {
         throw new UnsupportedOperationException("engineSetParameter unsupported");
      }
   }
}
